@@ -9,11 +9,10 @@ Created on Sat Apr 11 06:41:28 2020
 @author: Cyrile Delestre
 """
 
+import os
 import json
 import logging
 import sqlite3
-from os.path import exists
-from os import listdir
 from typing import List, Optional
 from datetime import datetime as dt
 
@@ -94,12 +93,14 @@ class HandlerBase(logging.Handler):
     --------
     SQLiteHandler, SQLAlchemyHandler
     """
-    def __init__(self,
-                 format: List[str]=['name', 'message']):
+    def __init__(
+        self,
+        format: List[str]=['name', 'message']
+    ) -> None:
         logging.Handler.__init__(self)
         self.format = format
 
-    def _asctime(self, record: logging.LogRecord):
+    def _asctime(self, record: logging.LogRecord) -> None:
         r"""
         Méthode ajoutant au LogRecord l'attribut asctime qui est le timestamp
         (created) du log au format lisible.
@@ -110,20 +111,20 @@ class HandlerBase(logging.Handler):
             .isoformat()
         )
 
-    def _message(self, record: logging.LogRecord):
+    def _message(self, record: logging.LogRecord) -> None:
         r"""
         Méthode ajoutant au LogRecord l'attribut message qui est une copie de 
         l'attribut msg au format json encodé en UTF-8.
         """
         record.message = json.dumps(record.msg).encode('utf-8')
 
-    def _engine(self, record: logging.LogRecord, engine: str):
+    def _engine(self, record: logging.LogRecord, engine: str) -> None:
         r"""
         Méthode ajoutant au LogRecord l'attribut engine.
         """
         record.engine = engine
 
-    def _extract_format(self, record: logging.LogRecord):
+    def _extract_format(self, record: logging.LogRecord) -> List[str]:
         r"""
         Méthode extrayant les informations du LogRecord sélectionnées par 
         l'attribut format.
@@ -132,7 +133,7 @@ class HandlerBase(logging.Handler):
         return list(map(lambda x: rec[x], self.format))
 
 
-    def emit(self, record: logging.LogRecord):
+    def emit(self, record: logging.LogRecord) -> None:
         r"""
         Méthode d'envoi des logs à implémenter.
         """
@@ -251,11 +252,13 @@ class SQLiteHandler(HandlerBase):
     --------
     SQLAlchemyHandler, timer, timer_class
     """
-    def __init__(self,
-                 path_sql: str,
-                 format: List[str]=['name', 'message'],
-                 base_name: str='logs',
-                 **kargs_sql):
+    def __init__(
+        self,
+        path_sql: str,
+        format: List[str]=['name', 'message'],
+        base_name: str='logs',
+        **kargs_sql
+    ) -> None:
         if 'msg' in format:
             raise AttributeError(
                 "Ne pas utiliser l'attribut 'msg' qui est technique, mais "
@@ -275,7 +278,7 @@ class SQLiteHandler(HandlerBase):
         conn.close()
         del conn
 
-    def emit(self, record: logging.LogRecord):
+    def emit(self, record: logging.LogRecord) -> None:
         self._asctime(record)
         self._message(record)
         self._engine(record, 'sqlite')
@@ -409,11 +412,13 @@ class SQLAlchemyHandler(HandlerBase):
     --------
     SQLiteHandler, timer, timer_class
     """
-    def __init__(self,
-                 path_sql: str,
-                 format: List[str]=['name', 'message'],
-                 base_name: str='logs',
-                 **kargs_sql):
+    def __init__(
+        self,
+        path_sql: str,
+        format: List[str]=['name', 'message'],
+        base_name: str='logs',
+        **kargs_sql
+    ) -> None:
         try:
             global sqlalchemy
             import sqlalchemy
@@ -449,7 +454,7 @@ class SQLAlchemyHandler(HandlerBase):
             )
         del engine
 
-    def emit(self, record: logging.LogRecord):
+    def emit(self, record: logging.LogRecord) -> None:
         self._asctime(record)
         self._message(record)
         self._engine(record, 'sqlalchemy')
@@ -527,16 +532,18 @@ class BaseArtimon(HandlerBase):
     --------
     ArtimonAPIHandler, ArtimonLocalHendleur
     """
-    def __init__(self, producer: str,
-                 format: List[str]=['name', 'msg'],
-                 hostname: Optional[str]=None,
-                 environment: Optional[str]=None):
+    def __init__(
+        self, producer: str,
+        format: List[str]=['name', 'msg'],
+        hostname: Optional[str]=None,
+        environment: Optional[str]=None
+    ) -> None:
         super().__init__(format = format)
         self.producer = producer
         self.hostname = hostname
         self.environment = environment
 
-    def parser(self, record: logging.LogRecord):
+    def parser(self, record: logging.LogRecord) -> str:
         name = record.name
         format = self.format
         if 'created' in self.format:
@@ -678,13 +685,15 @@ class ArtimonAPIHandler(BaseArtimon):
     --------
     ArtimonLocalHandler
     """
-    def __init__(self,
-                 producer: str,
-                 host: str='127.0.0.1',
-                 port: str='9023',
-                 format: List[str]=['name', 'msg'],
-                 hostname: Optional[str]=None,
-                 environment: Optional[str]=None):
+    def __init__(
+        self,
+        producer: str,
+        host: str='127.0.0.1',
+        port: str='9023',
+        format: List[str]=['name', 'msg'],
+        hostname: Optional[str]=None,
+        environment: Optional[str]=None
+    ) -> None:
         try:
             global requests
             import requests
@@ -713,7 +722,7 @@ class ArtimonAPIHandler(BaseArtimon):
             f"http://{self.host}:{self.port}/metrics"
         )
 
-    def emit(self, record: logging.LogRecord):
+    def emit(self, record: logging.LogRecord) -> None:
         self._asctime(record)
         self._engine(record, 'artimon')
         response = requests.post(
@@ -798,7 +807,7 @@ class ArtimonLocalHandler(BaseArtimon):
     environment : Optional[str]
         nom facultatif de l'environnement (si non renseigné sera complété 
         automatiquement par Artimon).
-    time_life : int
+    time_life : int (**deprecated**)
         nombre de jour d'activité du fichier récoltant les métriques. Le
         collecteur Artimon supprime automatiquement les fichiers qui ont 7 
         jours d'inactivité (par défaut 1 jours).
@@ -809,7 +818,10 @@ class ArtimonLocalHandler(BaseArtimon):
     il est important de ce référer à "msg" pour le message. De même il n'est 
     pas nécessaire de mettre "msg" dans l'argument format car le contenu du 
     message sera utilisé automatiquement comme argument de la série.
-    
+
+    L'argument time_life n'est plus pris en compte et sera supprimé dans les 
+    versions futures.
+
     Examples
     --------
     Envoi d'un timer mesurant de temps d'exécution d'une fonction et de sans 
@@ -855,13 +867,15 @@ class ArtimonLocalHandler(BaseArtimon):
     --------
     ArtimonAPIHandler
     """
-    def __init__(self,
-                 producer: str,
-                 path: str='/var/run/artimonV2',
-                 format: List[str]=['name', 'msg'],
-                 hostname: Optional[str]=None,
-                 environment: Optional[str]=None,
-                 time_life: int=1):
+    def __init__(
+        self,
+        producer: str,
+        path: str='/var/run/artimonV2',
+        format: List[str]=['name', 'msg'],
+        hostname: Optional[str]=None,
+        environment: Optional[str]=None,
+        #time_life: int=1
+    ) -> None:
         if 'message' in format:
             raise AttributeError(
                 "Ne pas utiliser l'attribut 'message' qui est technique, mais "
@@ -874,47 +888,54 @@ class ArtimonLocalHandler(BaseArtimon):
             environment = environment
         )
         self.path = path
-        self.time_life = time_life
-        if not exists(path):
+        # self.time_life = time_life
+        if not os.path.exists(path):
             raise FileNotFoundError(
                 f"Le répertoire {path} n'existe pas."
             )
 
-    def emit(self, record: logging.LogRecord):
+    def emit(self, record: logging.LogRecord) -> None:
         self._asctime(record)
         self._engine(record, 'artimon')
-        file_name = sorted(
-            filter(
-                lambda x: x.endswith('.artimon'),
-                listdir()
-            ),
-            reverse = True
-        )
-        if len(file_name) > 0:
-            file_name = file_name[0]
-            delta_j = (
-                dt.now().date()
-                -dt.strptime(file_name.split('.')[0], '%Y%m%d').date()
-            ).days
-            if delta_j > self.time_life:
-                file_name = f"{dt.now().strftime('%Y%m%d')}.artimon"
-        else:
-            file_name = f"{dt.now().strftime('%Y%m%d')}.artimon"
-        with open(file_name, 'a') as file:
+        file_name = f"{dt.now().strftime('%Y%m%d')}"
+        with open(f"{file_name}.tmp", 'w') as file:
             file.write(self.parser(record)+'\n')
+        os.rename(f"{file_name}.tmp", f"{file_name}.artimon")
+        # A remove si plus necessaire
+        # file_name = sorted(
+        #     filter(
+        #         lambda x: x.endswith('.artimon'),
+        #         os.listdir()
+        #     ),
+        #     reverse = True
+        # )
+        # if len(file_name) > 0:
+        #     file_name = file_name[0]
+        #     delta_j = (
+        #         dt.now().date()
+        #         -dt.strptime(file_name.split('.')[0], '%Y%m%d').date()
+        #     ).days
+        #     if delta_j > self.time_life:
+        #         file_name = f"{dt.now().strftime('%Y%m%d')}.artimon"
+        # else:
+        #     file_name = f"{dt.now().strftime('%Y%m%d')}.artimon"
+        # with open(file_name, 'a') as file:
+        #     file.write(self.parser(record)+'\n')
 
 class Warp10Handler(HandlerBase):
     r"""
     Classe de base permettant de créer un handler à partir d'une liste de 
     Handler vers une base interfacée à Warp10.
     """
-    def __init__(self,
-                 token: str,
-                 host: str='127.0.0.1',
-                 port: str='8080',
-                 field_value: str='value',
-                 format: List[str]=['name', 'msg'],
-                 api_version: str='v0'):
+    def __init__(
+        self,
+        token: str,
+        host: str='127.0.0.1',
+        port: str='8080',
+        field_value: str='value',
+        format: List[str]=['name', 'msg'],
+        api_version: str='v0'
+    ) -> None:
         try:
             global requests
             import requests
@@ -949,7 +970,7 @@ class Warp10Handler(HandlerBase):
             f"http://{self.host}:{self.port}/api/{self.api_version}/update"
         )
 
-    def emit(self, record: logging.LogRecord):
+    def emit(self, record: logging.LogRecord) -> None:
         self._asctime(record)
         self._engine(record, 'warp10')
         gts = GTS(record, self.field_value, self.format)
@@ -968,10 +989,12 @@ class GTS:
     r"""
     Classe modélisant un objet GTS (Geo Time Series) Warp10.
     """
-    def __init__(self,
-                 record: logging.LogRecord,
-                 dict_value: list,
-                 format: List[str]):
+    def __init__(
+        self,
+        record: logging.LogRecord,
+        dict_value: list,
+        format: List[str]
+    ) -> None:
         self.msg = record.msg.copy()
         field_value = dict_value[record.name]
         if isinstance(field_value, str):
@@ -995,7 +1018,7 @@ class GTS:
         log_att = record.__dict__
         self.log_att = list(map(lambda x: log_att[x], self.format))
 
-    def commit(self):
+    def commit(self) -> str:
         query = (
             f"{self.ts}// {self.name}"
             f"{{{','.join([f'{kk}={vv}' for kk, vv in zip(self.format, self.log_att)])}}}"
@@ -1012,5 +1035,4 @@ class GTS:
                 f"""[ {' '.join([(f"'{ii}'" if isinstance(ii, str) else f'{ii}') 
                 for ii in self.value])} ]"""
             )
-        print(query+value)
         return query+value
