@@ -18,28 +18,31 @@ from warnings import warn
 import numpy as np
 from sklearn.base import BaseEstimator
 import torch
-from torch.nn.functional import l1_loss
+from torch.optim import Optimizer
+from torch.nn.modules.loss import _Loss
+from torch.nn.utils import clip_grad_norm_
+from torch.cuda.amp import GradScaler, autocast
 from torch.utils.data import DataLoader, Dataset
 
 from dstk.pytorch import (check_tensor, CallbackInterface, CallbackHandler,
-                          FitState, FitControl, DEFAULT_CALLBACKS,
-                          StochasticWeightAveraging)
+                          FitState, FitControl, DEFAULT_CALLBACKS)
 from dstk.utils.errors import DivergenceError
 
 class BaseEnvironnement(BaseEstimator):
     r"""
     Classe d'environnement PyTorch pour être compatible Scikit-Learn.
-    
+
     Notes
     -----
     Les attributs constituant BaseEnvironnement :
-        dtype : 
-            type attendu en entrée du réseau (par défaut np.float32)
+        _is_fitted : bool
+            détermine si le modèle est entraîné
 
     Les méthodes constituant BaseEnvironnement :
-        set_dtype : 
-            définition du type PyTorch des données (par défaut float32)
-        save_weights : 
+        number_of_parameters : 
+            nombre de paramètre (entraînable ou non) constiuant la 
+            modélisation
+        save_model : 
             sauvegarde du modèle via Pickle
         load_weights : 
             chargement des poids via Pickle. Si modèle non buildé, build le 
@@ -73,8 +76,8 @@ class BaseEnvironnement(BaseEstimator):
             implémentée par l'utilisateur, ce sont alors les méthodes par 
             défaut dans Scikit-Learn qui seront utilisées, à savoir dans le 
             cas d'un Regressor, R^2, et dans le cas de Classifier, l'accuracy.
-    """ 
-    dtype = None
+    """
+    _is_fitted: bool = False
 
     def build(self):
         r"""
@@ -119,61 +122,14 @@ class BaseEnvironnement(BaseEstimator):
             model_parameters = self.parameters()
         return sum([np.prod(p.shape) for p in model_parameters])
 
-    def gpu(self, device: Optional[Union[int, torch.device]] = None):
-        r"""
-        Permet de mettre le modèle sur GPU en utilisant la méthode "cuda"
-        interne au module Module de PyTorch.
-
-        Parameters
-        ----------
-        device : Optional[Union[int, torch.device]] (=None)
-            Permet de préciser le divice si la machine est équipé de plusieurs 
-            GPU. Sinon envoit vers le GPU par défaut.
-
-        Warnings
-        --------
-        Pour mettre un réseau sur GPU afin de réaliser un apprentissage il 
-        faut dans un premier temps initialiser les poids du réseau et dans un 
-        second instancier l'optimizer. En effet, certain optimizer initialise 
-        des attribues importants pour l'éxécution de l'apprentissage dans 
-        leurs __init__. Si l'envoit vers le GPU ce fait après l'initialisation 
-        de l'optimizer une erreur ce produira (ceci est vrai que pour certain 
-        optimizer, la majorité fonctionneront même si le réseau a été envoyé 
-        dans un second temps).
-        """
-        self.cuda(device)
-        self.optimizer = self.optimizer.__class__(
-            self.parameters(),
-            **self.optimizer.defaults
-        )
-        return self
-
-    def set_dtype(self, dtype: torch.dtype):
-        r"""\
-        Méthode permettant de définir le type de Tensor dans PyTorch
-
-        Parameters
-        ----------
-        dtype : torch.dtype
-            Type du Tensor PyTorche.
-        """
-        if not isinstance(dtype, torch.dtype):
-            raise AttributeError(
-                "Le type dtype doit être de type torch.dtype et non de type "
-                f"{type(dtype)}."
-            )
-        self.dtype = dtype
-        parameters = self.state_dict()
-        for ii in parameters:
-            parameters[ii] = ii.type(dtype)
-        self.load_state_dict(parameters);
-        return self
-
-    def save_weights(self,
-                     path: Optional[str]=None,
-                     save_optimizer_state: bool=False,
-                     attr_list: Optional[Union[List[str], Tuple[str]]]=None,
-                     **kargs_dump):
+    def save_model(
+        self,
+        path: Optional[str]=None,
+        *,
+        save_optimizer_state: bool=False,
+        attr_list: Optional[Union[List[str], Tuple[str]]]=None,
+        **kargs_dump
+    ):
         r"""
         Méthode permettant de sauvegarder simplement les paramètres et les
         poids du modèle (sans la structure de réseau). Utilise Pickle.
@@ -231,10 +187,13 @@ class BaseEnvironnement(BaseEstimator):
                 "L'argument path doit être None ou un str et non de type "
                 f"{type(path)}."
             )
+        return self
 
-    def load_weights(self, 
-                     path_or_bytes: Union[str, ByteString],
-                     **kargs_load):
+    def load_weights(
+        self, 
+        path_or_bytes: Union[str, ByteString],
+        **kargs_load
+    ):
         r"""
         Méthode permettant de charger les poids et les paramètres du modèle
         via Pickle.
@@ -278,9 +237,11 @@ class BaseEnvironnement(BaseEstimator):
         return self
 
     @classmethod
-    def load_model(cls,
-                   path_or_bytes: Union[str, ByteString],
-                   **kargs_load):
+    def load_model(
+        cls,
+        path_or_bytes: Union[str, ByteString],
+        **kargs_load
+    ):
         r"""
         Fonction permettant de charger un modèle à partir d'une sauvegarde au
         format Pickle issue de save_weights(path).
@@ -350,7 +311,7 @@ class BaseEnvironnement(BaseEstimator):
         """
         if np.isnan(loss) or np.isinf(loss):
             raise DivergenceError(
-                """Verger ! L'algorithme à dit "verger" !"""
+                """Verger ! L'algorithme a dit "verger" !"""
             )
 
     def _mod(self, n, mod):
@@ -362,11 +323,15 @@ class BaseEnvironnement(BaseEstimator):
         else:
             return None
 
-    def _train_step(self,
-                    data: List[Dict[str, Any]],
-                    field_target: str,
-                    loss_fn: Callable,
-                    kargs_loss: Dict[str, Any]):
+    def _train_step(
+        self,
+        data: List[Dict[str, Any]],
+        loss_fn: Callable,
+        loss_kargs: Dict[str, Any],
+        state : FitState,
+        control: FitControl,
+        list_callbacks: List[CallbackInterface]
+    ):
         r"""
         Méthode privée d'une étape de backpropagation du gradient.
 
@@ -374,34 +339,58 @@ class BaseEnvironnement(BaseEstimator):
         ----------
         data : List[Dict[str, Any]]
             Liste de dictionnaires sur les données d'un mini-batch.
-        field_target : str
-            Nom du champs de la target.
         loss_fn : Callable
             Fonction coût PyTorch.
-        kargs_loss : Dict[str, Any]
+        loss_kargs : Dict[str, Any]
             Dictionnaire d'arguments à utiliser dans la fonction de coût 
             loss_fun.
+        state : FitState
+            Etat de l'apprentissage (instancié dans la méthode 
+            :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
+        control : FitControl
+            Contrôle de l'apprentissage (instancié dans la méthode 
+            :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
+        list_callbacks : List[CallbackInterface]
+            Liste des callback permettant de contôler l'apprentissage durant 
+            la phase d'évaluation.
 
         Returns
         -------
         loss : Valeur de la fonction coût au travers du mini-batch.
         """
+        list_callbacks.begin_step(self, data, state, control, None)
         self.train()
-        self.optimizer.zero_grad()
-        output = self.forward(**data)
-        target = check_tensor(data[field_target])
-        loss_ = loss_fn(output, target, **kargs_loss)
-        loss_.backward()
-        self.optimizer.step()
+        if control.device.type != "cpu":
+            for key, value in data.items():
+                if key not in control.expected_key_to_device:
+                    data[key] = value.to(control.device)
+        with autocast(enabled=control.mixed_type_gpu):
+            output = self.forward(**data)
+            target = check_tensor(data[control.target_field])
+            loss_ = loss_fn(output, target, **loss_kargs)
+        if control.gradient_accumulation > 1:
+            loss_ /= control.gradient_accumulation
+        control.grad_scaler.scale(loss_).backward()
+        state.n_iter += 1
+        if state.n_iter % control.gradient_accumulation == 0:
+            if control.max_gradient_norm:
+                control.grad_scaler.unscale_(self.optimizer)
+                clip_grad_norm_(self.parameters(), control.max_gradient_norm)
+            control.grad_scaler.step(self.optimizer)
+            control.grad_scaler.update()
+            self.optimizer.zero_grad()
         loss = loss_.detach()
         self._divergence(loss)
+        list_callbacks.end_step(self, data, state, control, loss)
         return loss
 
-    def _eval_step(self,
-                   data: List[Dict[str, Any]],
-                   field_target: str,
-                   loss_fn: Callable,
-                   kargs_loss: Dict[str, Any]):
+    def _eval_step(
+        self,
+        data: List[Dict[str, Any]],
+        loss_fn: Callable,
+        loss_kargs: Dict[str, Any],
+        control : FitControl
+    ):
         r"""
         Méthode privée d'une étape deévaluation.
 
@@ -409,35 +398,41 @@ class BaseEnvironnement(BaseEstimator):
         ----------
         data : List[Dict[str, Any]]
             Liste de dictionnaires sur les données d'un mini-batch.
-        field_target : str
-            Nom du champs de la target.
         loss_fn : Callable
             Fonction coût PyTorch.
-        kargs_loss : Dict[str, Any]
+        loss_kargs : Dict[str, Any]
             Dictionnaire d'arguments à utiliser dans la fonction de coût 
             loss_fun.
+        control : FitControl
+            Contrôle de l'apprentissage (instancé dans la méthode 
+            :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
 
         Returns
         -------
         loss : Valeur de la fonction coût au travers du mini-batch.
         """
         self.eval()
+        if control.device.type != "cpu":
+            for key, value in data.items():
+                if key not in control.expected_key_to_device:
+                    data[key] = value.to(control.device)
         with torch.no_grad():
             output = self.forward(**data)
-            target = check_tensor(data[field_target])
-            loss = loss_fn(output, target, **kargs_loss)
+            target = check_tensor(data[control.target_field])
+            loss = loss_fn(output, target, **loss_kargs)
         loss = loss.detach()
         self._divergence(loss)
         return loss
 
-    def _iter_eval(self,
-                   eval_dataset: DataLoader,
-                   field_target: str,
-                   loss_fn: Callable,
-                   kargs_loss: Optional[Dict[str, Any]],
-                   state: FitState,
-                   control: FitControl,
-                   list_callbacks: List[CallbackInterface]):
+    def _iter_eval(
+        self,
+        eval_dataset: DataLoader,
+        loss_fn: Callable,
+        loss_kargs: Optional[Dict[str, Any]],
+        state: FitState,
+        control: FitControl,
+        list_callbacks: List[CallbackInterface]
+    ):
         r"""
         Méthode interne exécutant la phase d'évaluation durant l'apprentissage.
         Cette étape peut être appelé à la fin d'une époque ou durant 
@@ -449,8 +444,6 @@ class BaseEnvironnement(BaseEstimator):
         eval_dataset : DataLoader
             Dataset d'évaluation au format DataLoeader généré dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`.
-        field_target : str
-            Nom du champs target.
         loss_fn : Callable
             Fonction coût à minimiser (utiliser les fonctions présentes dans 
             torch.nn.functional). Si la fonction est une fonction custom, elle 
@@ -458,7 +451,7 @@ class BaseEnvironnement(BaseEstimator):
                 - loss_fn(y_estim, y_true, reduction='sum', **kargs)
             avec une implémentation de réduction 'sum' pour les phases 
             d'évaluation et 'mean' pour les phases d'apprentissage.
-        kargs_loss : Optional[Dict[str, Any]]
+        loss_kargs : Optional[Dict[str, Any]]
             Dictionnaire d'arguments à utiliser dans la fonction de coût 
             loss_fun.
         state : FitState
@@ -490,9 +483,9 @@ class BaseEnvironnement(BaseEstimator):
             try:
                 loss_eval += self._eval_step(
                     data=eval_batch,
-                    field_target=field_target,
                     loss_fn=loss_fn,
-                    kargs_loss=kargs_loss
+                    loss_kargs=loss_kargs,
+                    control=control
                 )
             except DivergenceError as error:
                 control.diverge = True
@@ -506,88 +499,105 @@ class BaseEnvironnement(BaseEstimator):
             res_loss=loss_eval/(jj+1)
         )
 
-    def fit(self,
-            X: Union[List[Dict[str, Any]], Dataset, DataLoader],
-            y: Optional[List[Union[int, float]]]=None,
-            eval_set: Optional[Union[List[Dict[str, Any]], Dataset]]=None,
-            nb_epoch: int=5,
-            iter_eval: Optional[int]=None,
-            collate_fn: Optional[Callable]=None,
-            loss_fn: Callable=l1_loss,
-            kargs_loss: Dict[str, Any]=dict(),
-            field_target: str='target',
-            dataloader: bool=True,
-            batch_size: int=32,
-            shuffle: bool=True,
-            drop_last: bool=False,
-            num_workers: int=0,
-            num_threads: Optional[int]=None,
-            callbacks: List[CallbackInterface]=DEFAULT_CALLBACKS,
-            swa_module: Optional[StochasticWeightAveraging]=None,
-            rebuild: bool=False):
+    def fit(
+        self,
+        X: Union[List[Dict[str, Any]], Dataset, DataLoader],
+        y: Optional[List[Union[int, float]]]=None,
+        *,
+        eval_dataset: Optional[DataLoader]=None,
+        target_field: str='target',
+        loss_fn: _Loss,
+        loss_kargs: Dict[str, Any]=dict(),
+        optimizer: Optional[Optimizer]=None,
+        optimizer_kargs: Dict[str, any]=dict(),
+        nb_epoch: int,
+        iter_eval: Optional[int]=None,
+        callbacks: List[CallbackInterface]=DEFAULT_CALLBACKS,
+        device: str='cuda:0',
+        mixed_type_gpu: bool=False,
+        expected_key_to_device: List[str]=list(),
+        gradient_accumulation: Optional[int]=None,
+        max_gradient_norm: Optional[float]=None,
+        dataloader_kargs: Dict[str, Any]=dict(),
+        n_jobs: Optional[int]=None,
+        state: Optional[FitState]=None,
+        control: Optional[FitControl]=None,
+        build: bool=True
+    ):
         r"""
+        Méthode de fitting généralisé du module PyTorch.
+        
+        Parameters
+        ----------
         X : Union[List[Dict[str, Any]], Dataset, DataLoader]
             Dataset PyTorch contenant dans un dictionnaire les infos 
             nécessaires pour l'apprentissage du modèle.
-        y : Optional[List[Union[int, float]]] (=None)
+        y : Optional[List[Union[int, float]]] 
             Pour être ISO avec Scikit-learn. L'ensemble de l'apprentissage
             passera via X et le système DataSet et DataLoader de PyTorch.
-        eval_set : Optional[Union[List[Dict[str, Any]], Dataset]] (=None)
-            Dataset PyTorch contenant les observations et les targets 
-            d'évaluation. Si eval_set est None alors pas de early stopping 
+        eval_dataset : Optional[DataLoader]
+            DataLoader PyTorch contenant les observations et les targets 
+            d'évaluation. Si eval_dataset est None alors pas de early stopping 
             et évaluation des performances sur les données d'entrainement 
             (déconseillé car très fort risque d'overfitting).
-        nb_epoch : int (=5)
-            Nombre d'époques max de l'apprentissage.
-        iter_eval : Optional[int] (=None)
-            Nombre d'itération entre deux évaluations. Actif seulement si 
-            eval_set est non None. Si iter_eval est à None (définit par 
-            défaut) les évaluations du modèle seront à la fin des époques. 
-            Si un entier positif est renseigné les évaluations auront lieu 
-            tous les iter_eval itérations d'apprentissage.
-        collate_fn : Optional[Callable] (=None)
-            Fonction de sortie de DataLoader permettant la concaténation en 
-            batch des différentes observations du mini-batch.
-        loss_fn : Optional[Callable] (=None)
+        target_field : str (='target')
+            Nom du champ contenu dans X de la target. Par défaut "target". 
+            Attention l'implémentation de la foction collate_fn, peut faire 
+            changer le nom de la target.
+        loss_fn : _Loss
             Fonction coût à minimiser (utiliser les fonctions présentes dans 
             torch.nn.functional). Si la fonction est une fonction custom, elle 
             doit avoir le prototypage suivant :
                 - loss_fn(y_estim, y_true, reduction='sum', **kargs)
             avec une implémentation de réduction 'sum' pour les phases 
             d'évaluation et 'mean' pour les phases d'apprentissage.
-        kargs_loss : Dict[str, Any] (=dict())
+        loss_kargs : Dict[str, Any]
             Dictionnaire d'arguments à utiliser dans la fonction decoût 
             loss_fun. Si il n'y a pas d'arguments, mettre un dictionnaire 
             vide.
-        field_target : str (='target')
-            Nom du champ contenu dans X de la target. Par défaut "target". 
-            Attention l'implémentation de la foction collate_fn, peut faire 
-            changer le nom de la target.
-        dataloader : bool (=True)
-            Indique si la donnée X sera transformée en DataLoader ou non. Par 
-            défaut True.
-        batch_size : int (=32)
-            taille du mini-batch. Pris en compte seulement si dataloader = 
-            True.
-        shuffle : bool (=True)
-            Permet de mélanger à chaque époque les observations. Pris en 
-            compte seulement si dataloader = True.
-        drop_last : bool (=False)
-            Drop les dernières observations si le rapport entre le nombre 
-            d'observations et la taille du mini-batch est non entier. Pris en 
-            compte seulement si dataloader = True.
-        num_workers : int (=0)
-            Nombre de process du DataLoader. Par défaut 0 correspondant à du 
-            mono-thread. Pris en compte seulement si dataloader = True.
-                - Attention :
-                    1 -> mono-threadé mais isolé de l'environnement 
-                    (Picklelisé). Si le DataLoader est utilisé dans un 
-                    environnement Sciki-Learn multi-processing (par exemple 
-                    pour un RandomizedSearchCV) il est très important de 
-                    mettre cet argument à 0 pour que la DataLoader tourne 
-                    dans l'environnement principale de Python. Dans le cas 
-                    contraire le processus plantera.
-        num_threads : Optional[int] (=None)
+        optimizer : Optional[Optimizer]
+            Objet optimizer de PyTorch. Si non renseigné doit être implémenté 
+            dans le modèle et dans la partie build. Utile si un paramètre de 
+            l'optimizer doit faire partie des hyperparamètres. Sinon à 
+            renseigner ici pour alléger l'implémentation globale.
+        optimizer_kargs: Dict[str, any]
+            Paramètres pour l'optimizer, si optimizer inclue dans la 
+            modélisation et l'argument optimizer est à None cet argument ne 
+            sera pas pris en compte.
+        nb_epoch : int (=5)
+            Nombre d'époques max de l'apprentissage.
+        iter_eval : Optional[int]
+            Nombre d'itération entre deux évaluations. Actif seulement si 
+            eval_set est non None. Si iter_eval est à None (définit par 
+            défaut) les évaluations du modèle seront à la fin des époques. 
+            Si un entier positif est renseigné les évaluations auront lieu 
+            tous les iter_eval itérations d'apprentissage.
+        callbacks : List[CallbackInterface] (=DEFAULT_CALLBACKS)
+            Liste de callback intervenant aux différentes étapes de 
+            l'entraînement et de l'évaluation. Pour avoir la liste des 
+            différents callbacks implémentés par défaut voir 
+            :func:`~dstk.pytorch._callback`. Par défaut utilise les 
+            DEFAULT_CALLBACKS.
+        device : str (='cuda:0')
+            Indique le device sur le quel l'apprentissage s'exécute.
+        mixed_type_gpu : bool (=False)
+            Utilisation de nVidia AMP (Automatic Mixed Precision) pour GPU 
+            permettant le typage mixte FP16 et FP32.
+        expected_key_to_device : List[str] (=list())
+            Liste des variables à exclure vers la charge sur GPU.
+        gradient_accumulation : Optional[int]
+            Nombre d'itération d'entrainement avec back propagation du 
+            gradient avant d'appliquer la descente de gradient.
+        max_gradient_norm : Optional[float]
+            Valeur maximum du gradient, sinon clipping de ce dernier par la 
+            valeur max_gradient_norm. Si non renseigné alors pas de gradient 
+            clipping. Le clipping du gradient peut être utilie pour l'approche 
+            par accumulation des gradient.
+        dataloader_kargs : Dict[str, Any] (=dict())
+            Si X n'est pas un objet Dataloader permet d'entrer les arguments 
+            du Dataloader (voir la `documention PyTorch_ `pour connaitre les 
+            arguments du Dataloader).
+        n_jobs : Optional[int]
             Nombre de process des étapes d'inférence et de backpropagation des 
             gradients :
                 - Si y est not None :
@@ -597,17 +607,13 @@ class BaseEnvironnement(BaseEstimator):
                     (et pas le nombre de threads) ;
                 - Si un entier :
                     utilise le nombre de processus indiqué.
-        callbacks : List[CallbackInterface] (=DEFAULT_CALLBACKS)
-            Liste de callback intervenant aux différentes étapes de 
-            l'entraînement et de l'évaluation. Pour avoir la liste des 
-            différents callbacks implémentés par défaut voir 
-            :func:`~dstk.pytorch._callback`. Par défaut utilise les 
-            DEFAULT_CALLBACKS.
-        swa_module : Optional[StochasticWeightAveraging] (=None)
-            module permettant d'appliquer l'approche 
-            `Stochastic Weight Averaging (SWA)`_. Voir la classe 
-            :mod:`~dstk.pytorch._swa.StochasticWeightAveraging`.
-        rebuild : bool (=False)
+        state : Optional[FitState]
+            Etat de l'apprentissage (instancié dans la méthode 
+            :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
+        control : Optional[FitControl]
+            Contrôle de l'apprentissage (instancé dans la méthode 
+            :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
+        build : bool (=True)
             Permet de reconstruire le réseau si, entre la définition de la 
             classe et le fit, il y a changement d'hyper-paramètres (exemple : 
             RandomizedSearchCV ou toute autre fonction Scikit-learn pouvant 
@@ -621,20 +627,15 @@ class BaseEnvironnement(BaseEstimator):
         fit, il est conseillé d'utiliser le type DataSet PyTorch comme type 
         d'entré à X.
 
-        Notes
-        -----
-        Pour le moment l'apprentissage sur GPU n'est pas pris en compte dans 
-        où la fonction fit est appelé par un module Scikit-Learn (comme 
-        RandomizedSearchCV par exemple).
-
-        .. _Stochastic Weight Averaging (SWA): https://arxiv.org/abs/1803.05407
+        .. _documention PyTorch: https://pytorch.org/docs/stable/data.html
         """
-        if not hasattr(self, 'optimizer'):
+        if not hasattr(self, 'optimizer') and optimizer is None:
             raise NotImplementedError(
                 "L'estimateur ne possède par d'objet 'optimizer'. Cette "
                 "attribue doit être un objet d'otimisation de type PyTorch : "
                 "torch.optim.(SGD, ADAM, etc.). Il faut le déclarer dans "
-                "la construction de la classe modèle."
+                "la construction de la classe modèle ou le définir dans "
+                "les arguments de la méthode fit."
             )
 
         if iter_eval is not None and iter_eval < 0:
@@ -642,173 +643,155 @@ class BaseEnvironnement(BaseEstimator):
                 "iter_eval doit être soit None ou un entier positif."
             )
 
-        if swa_module:
-            if not isinstance(swa_module, StochasticWeightAveraging):
-                raise AttributeError(
-                    "swa_module doit être de type StochasticWeightAveraging "
-                    f"et non de type {type(swa_module).__name__}."
+        if (
+                gradient_accumulation is not None and
+                iter_eval < gradient_accumulation
+        ):
+            warn(
+                "iter_eval < gradient_accumulation donc à chaque itération "
+                "un pas de gradient ne sera pas effectué."
+            )
+
+        if optimizer is not None:
+            if hasattr(self, 'optimizer'):
+                warn(
+                    "Un optimizer a été donné en argument de la méthode fit "
+                    "hors il en existait déjà un attribue optimizer dans la "
+                    "classe du modèle. La priorité est accordé à celui en "
+                    "entré du fit et à donc écrasé celui du modèle de base."
                 )
-            swa_module = swa_module.copy()
+            self.optimizer = optimizer(self.parameters(), **optimizer_kargs)
 
         list_callbacks = CallbackHandler(callbacks)
         list_callbacks.copy()
 
         id_processes = str(uuid.uuid1())
 
-        # Si y est non None c'est que le processes passe par un search de
-        # Scikit-Learn.
-        if y is not None:
-            torch.set_num_threads(1)
-            num_workers = 0
-            rebuild = True
-        elif num_threads is not None:
-            torch.set_num_threads(num_threads)
-        else:
-            torch.set_num_threads(torch.get_num_threads())
-
-        if rebuild:
+        # Permet de construire le modèle avec les paramètres actuels du fit.
+        if build:
             self.build()
 
-        if isinstance(X, dict):
-            X = [X]
-
-        if swa_module:
-            X = swa_module.cut_dataset(
-                dataset=X,
-                field_target=field_target
-            )
-
-        if dataloader and not isinstance(X, DataLoader):
+        if not isinstance(X, DataLoader):
+            if isinstance(X, dict):
+                X = [X]
             dataset = DataLoader(
                 dataset=X,
-                collate_fn=collate_fn,
-                shuffle=shuffle,
-                drop_last=drop_last,
-                batch_size=batch_size,
-                num_workers=num_workers
+                **dataloader_kargs
             )
         else:
             dataset = X
 
-        if eval_set:
-            eval_dataset = DataLoader(
-                dataset=eval_set,
-                collate_fn=collate_fn,
-                shuffle=False,
-                drop_last=False,
-                batch_size=16,
-                num_workers=num_workers
-            )
+        # Si y est non None c'est que le processes passe par un search de
+        # Scikit-Learn.
+        if y is not None:
+            torch.set_num_threads(1)
+            dataset.num_workers = 0
+        elif n_jobs is not None:
+            torch.set_num_threads(n_jobs)
         else:
-            eval_dataset = None
+            torch.set_num_threads(torch.get_num_threads())
 
-        state = FitState(
-            id_process=id_processes,
-            nb_epoch=nb_epoch,
-            epoch_max_steps=len(dataset),
-            eval_max_steps=len(eval_dataset) if eval_dataset else 0,
-            loss_name=(
-                loss_fn.__name__ if isinstance(loss_fn, FunctionType)
-                else loss_fn.__class__.__name__
+        if not state:
+            state = FitState(
+                id_process=id_processes,
+                nb_epoch=nb_epoch,
+                epoch_max_steps=len(dataset),
+                eval_max_steps=len(eval_dataset) if eval_dataset else 0,
+                loss_name=(
+                    loss_fn.__name__ if isinstance(loss_fn, FunctionType)
+                    else loss_fn.__class__.__name__
+                ),
+                n_iter=0
             )
-        )
-        control = FitControl()
+        if not control:
+            control = FitControl(
+                target_field=target_field,
+                device=(
+                    device := torch.device(
+                        device if torch.cuda.is_available() else 'cpu'
+                    )
+                ),
+                mixed_type_gpu=mixed_type_gpu and device.type != "cpu",
+                grad_scaler=GradScaler(
+                    enabled=(mixed_type_gpu and device.type != "cpu")
+                ),
+                expected_key_to_device=expected_key_to_device,
+                gradient_accumulation=gradient_accumulation,
+                max_gradient_norm=max_gradient_norm
+            )
+        self.to(control.device)
 
         list_callbacks.begin_train(self, dataset, state, control, None)
-        for n_epoch in range(nb_epoch):
-            state.n_epoch = n_epoch
-            loss_epoch = 0
-            list_callbacks.begin_epoch(
-                model=self,
-                data=dataset,
-                state=state,
-                control=control,
-                res_loss=loss_epoch
-            )
-            for ii, batch in enumerate(dataset):
-                state.n_iter = ii
-
-                if (
-                    eval_dataset and
-                    self._mod(n_epoch*state.epoch_max_steps+ii, iter_eval) == 0
-                ):
+        try:
+            for n_epoch in range(nb_epoch):
+                state.n_epoch = n_epoch
+                loss_epoch = 0
+                list_callbacks.begin_epoch(
+                    model=self,
+                    data=dataset,
+                    state=state,
+                    control=control,
+                    res_loss=loss_epoch
+                )
+                for ii, batch in enumerate(dataset):
+                    if (
+                        eval_dataset and
+                        self._mod(state.n_iter, iter_eval) == 0
+                    ):
+                        self._iter_eval(
+                            eval_dataset=eval_dataset,
+                            loss_fn=loss_fn,
+                            loss_kargs=loss_kargs,
+                            state=state,
+                            control=control,
+                            list_callbacks=list_callbacks
+                        )
+                        if control.diverge or control.training_stop:
+                            break
+                    try:
+                        loss_step = self._train_step(
+                            data=batch,
+                            loss_fn=loss_fn,
+                            loss_kargs=loss_kargs,
+                            state=state,
+                            control=control,
+                            list_callbacks=list_callbacks
+                        )
+                        loss_epoch += loss_step
+                    except DivergenceError as error:
+                        control.diverge = True
+                        warn(error.msg)
+                        break
+                loss_epoch /= (ii+1)
+                list_callbacks.end_epoch(
+                    model=self,
+                    data=dataset,
+                    state=state,
+                    control=control,
+                    res_loss=loss_epoch
+                )
+                if eval_dataset and iter_eval is None:
                     self._iter_eval(
                         eval_dataset=eval_dataset,
-                        field_target=field_target,
                         loss_fn=loss_fn,
-                        kargs_loss=kargs_loss,
+                        loss_kargs=loss_kargs,
                         state=state,
                         control=control,
                         list_callbacks=list_callbacks
                     )
-                    if control.diverge or control.training_stop:
-                        break
-
-                list_callbacks.begin_step(self, batch, state, control, None)
-                try:
-                    loss_step = self._train_step(
-                        data=batch,
-                        field_target=field_target,
-                        loss_fn=loss_fn,
-                        kargs_loss=kargs_loss
-                    )
-                    loss_epoch += loss_step
-                    list_callbacks.end_step(
-                        model=self,
-                        data=batch,
-                        state=state,
-                        control=control,
-                        res_loss=loss_step
-                    )
-                except DivergenceError as error:
-                    control.diverge = True
-                    warn(error.msg)
+                if control.diverge or control.training_stop:
                     break
-            loss_epoch /= (ii+1)
-            list_callbacks.end_epoch(self, dataset, state, control, loss_epoch)
-
-            if eval_dataset and iter_eval is None:
-                self._iter_eval(
-                    eval_dataset=eval_dataset,
-                    field_target=field_target,
-                    loss_fn=loss_fn,
-                    kargs_loss=kargs_loss,
-                    state=state,
-                    control=control,
-                    list_callbacks=list_callbacks
-                )
-            if control.diverge or control.training_stop:
-                break
-
-        list_callbacks.end_train(self, dataset, state, control, None)
-
-        if swa_module and not control.diverge:
-            control.training_stop = False
-            res_loss = loss_epoch
-            if eval_dataset:
-                early = [
-                    ii for ii in list_callbacks.callbacks
-                    if hasattr(ii, 'old_res_loss')
-                ]
-                if len(early) == 1:
-                    res_loss = early[0].old_res_loss
-            swa_module.init(
+        except (Exception, KeyboardInterrupt) as e:
+            list_callbacks.error_interrupt(
                 model=self,
-                res_loss=res_loss
-            )
-            swa_module.fit(
-                model=self,
-                eval_dataset=eval_dataset,
-                loss_fn=loss_fn,
-                kargs_loss=kargs_loss,
-                field_target=field_target,
-                iter_eval=iter_eval,
-                collate_fn=collate_fn,
-                batch_size=batch_size,
-                shuffle=shuffle,
-                drop_last=drop_last,
-                num_workers=num_workers,
+                data=dataset,
                 state=state,
-                control=control
+                control=control,
+                res_loss=None,
+                exception=e
             )
+            raise
+        list_callbacks.end_train(self, dataset, state, control, None)
+        if control.device.type != "cpu": self.to(torch.device('cpu:0'))
+        self._is_fitted = True
         return self
